package svc

import (
	"applet/app/admin/svc/enterprise_manage"
	"applet/app/customer/md"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/enum"
	"applet/app/utils"
	"errors"
	"fmt"
	"time"
)

func NursingHomeOrderList(req md.NursingHomeOrderListReq) (m []model.NursingHomePackageOrd, total int64, err error) {
	sess := db.Db.Desc("id").Where("user_identity_id =?", req.UserIdentityId).
		And("create_at >= ? And create_at <= ?", req.StartDate, req.EndDate).Limit(req.Limit, (req.Page-1)*req.Limit)
	sess.And("ord_state = ?", req.OrdState)
	total, err = sess.FindAndCount(&m)
	if err != nil {
		return
	}
	return
}

func NursingHomeOrderStatistic(req md.NursingHomeOrderListReq) (total float64, err error) {
	var sql = fmt.Sprintf("SELECT SUM(total_price) AS sum_total FROM nursing_home_package_ord where "+
		"user_identity_id = '%d' and create_at >= '%s' And create_at <= '%s' and ord_state = '%d'", req.UserIdentityId, req.StartDate, req.EndDate, req.OrdState)
	mapArr1, err := db.QueryNativeString(db.Db, sql)
	if err != nil {
		return
	}
	if mapArr1 == nil {
		total = 0
	} else {
		total = utils.StrToFloat64(mapArr1[0]["sum_total"])
	}
	return
}

func NursingHomeOrderRefund(req md.NursingHomeOrderRefundReq) (err error) {
	sysCfgDb := db.SysCfgDb{}
	sysCfgDb.Set()
	cfg, err := sysCfgDb.SysCfgGetOne(enum.NursingHomeReserveMealTime)
	if err != nil {
		return
	}

	NursingHomeUserWithDayDb := db.NursingHomeUserWithDayDb{}
	NursingHomeUserWithDayDb.Set(0)
	NursingHomeUserWithDay, err := NursingHomeUserWithDayDb.GetNursingHomeUserWithDay(req.Id)
	if err != nil {
		return err
	}
	if NursingHomeUserWithDay == nil {
		err = errors.New("未查询到订餐记录")
		return
	}
	//1、判断是否为“待就餐”状态
	if NursingHomeUserWithDay.State != enum.NursingHomeUserWithDayStateForWait {
		err = errors.New("当前订餐不可退订")
		return
	}

	//2、判断是否为可退餐时间段
	now := time.Now()
	today, _ := time.ParseInLocation("2006-01-02", time.Now().Format("2006-01-02"), time.Local)
	NursingHomeReserveMealTime, _ := time.ParseInLocation("2006-01-02 15:04:05", now.Format("2006-01-02")+" "+cfg.Val+":00", time.Local)
	date, _ := time.ParseInLocation("2006-01-02", NursingHomeUserWithDay.Date, time.Local)
	if today.Equal(date) {
		//2.2、判断是否过了今日可订餐时间
		if now.After(NursingHomeReserveMealTime) {
			err = errors.New("当前订餐不在可退订时间范围内")
			return
		}
	}

	//3、处理订餐记录状态
	NursingHomeUserWithDay.State = enum.NursingHomeUserWithDayStateForCanceling
	updateAck, err := NursingHomeUserWithDayDb.NursingHomeUserWithDayUpdate(NursingHomeUserWithDay.Id, NursingHomeUserWithDay, "state")
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		err = errors.New("更新订餐记录状态失败")
		return
	}

	//4、处理订单状态
	err = svc.JudgeNursingHomePackageOrdOrdState(req.OutTradeNo)
	if err != nil {
		return err
	}

	//5、新增退款记录 nursing_home_user_refund_day
	outRequestNo := utils.OrderUUID(NursingHomeUserWithDay.Uid)
	NursingHomeUserRefundDayDb := db.NursingHomeUserRefundDayDb{}
	NursingHomeUserRefundDayDb.Set(0)
	_, err = NursingHomeUserRefundDayDb.NursingHomeUserRefundDayInsert(&model.NursingHomeUserRefundDay{
		OutTradeNo:   req.OutTradeNo,
		OutRequestNo: outRequestNo,
		Uid:          NursingHomeUserWithDay.Uid,
		IdentityId:   NursingHomeUserWithDay.IdentityId,
		RecordsId:    NursingHomeUserWithDay.Id,
		State:        enum.NursingHomeUserRefundDayStateForAuditing,
		Amount:       NursingHomeUserWithDay.Amount,
		Memo:         "",
		CreateAt:     now.Format("2006-01-02 15:04:05"),
		UpdateAt:     now.Format("2006-01-02 15:04:05"),
	})
	return
}

func NursingHomeOrderRefundList(req md.NursingHomeOrderRefundListReq) (resp []md.NursingHomeOrderRefundListResp, total int64, err error) {
	var m []*db.NursingHomeUserRefundDayWithNursingHomeUserWithDay
	sess := db.Db.Where("nursing_home_user_refund_day.identity_id =?", req.UserIdentityId)
	if req.State != 0 {
		sess = sess.And("nursing_home_user_refund_day.state = ?", req.State)
	}
	if req.StartDate != "" {
		sess = sess.And("nursing_home_user_refund_day.create_at >= ?", req.StartDate)
	}
	if req.EndDate != "" {
		sess = sess.And("nursing_home_user_refund_day.create_at <= ?", req.EndDate)
	}
	total, err = sess.Desc("nursing_home_user_refund_day.id").
		Join("LEFT", "nursing_home_user_with_day", "nursing_home_user_refund_day.records_id = nursing_home_user_with_day.id").
		Limit(req.Limit, (req.Page-1)*req.Limit).FindAndCount(&m)
	if err != nil {
		return nil, 0, err
	}
	userIdentityDb := db.UserIdentityDb{}
	userIdentityDb.Set(req.UserIdentityId)
	identity, err := userIdentityDb.UserIdentity(req.UserIdentityId)
	if err != nil {
		return nil, 0, err
	}

	for _, v := range m {
		resp = append(resp, md.NursingHomeOrderRefundListResp{
			OutTradeNo:     v.NursingHomeUserRefundDay.OutTradeNo,
			OutRequestNo:   v.NursingHomeUserRefundDay.OutRequestNo,
			Kind:           v.NursingHomeUserWithDay.Kind,
			CreateAt:       v.NursingHomeUserRefundDay.CreateAt,
			RefundDate:     v.NursingHomeUserRefundDay.RefundDate,
			Date:           v.NursingHomeUserWithDay.Date,
			State:          v.NursingHomeUserRefundDay.State,
			Memo:           v.NursingHomeUserRefundDay.Memo,
			Amount:         v.NursingHomeUserRefundDay.Amount,
			EnterpriseName: identity.Enterprise.Name,
		})
	}
	return
}

func NursingHomeMyReserve(userIdentityId int, date, startDate, endDate string) (err error, resp md.NursingHomeMyReserveResp) {
	resp.Date = date
	//1、查询出对应时间段，所有 "待就餐"、 "已就餐" 记录
	NursingHomeUserWithDayDb := db.NursingHomeUserWithDayDb{}
	NursingHomeUserWithDayDb.Set(userIdentityId)
	list, err := NursingHomeUserWithDayDb.FindNursingHomeUserWithDayByDate(startDate, endDate)
	if err != nil {
		return
	}

	//2、构造数据
	var totalBreakfast, totalLunch, totalDinner, total int
	var dateMap = map[string]*md.NursingHomeMyReserveRespList{}
	var dateMapKey []string
	for _, v := range *list {
		if v.State == enum.NursingHomeUserWithDayStateForWait || v.State == enum.NursingHomeUserWithDayStateForAlready || v.State == enum.NursingHomeUserWithDayStateForCanceling {
			if dateMap[v.Date] == nil {
				dateMapKey = append(dateMapKey, v.Date)
				dateMap[v.Date] = &md.NursingHomeMyReserveRespList{}
			}
			dateMap[v.Date].Date = v.Date
			total++
			if v.Kind == enum.NursingHomeUserWithDayKindForBreakfast {
				totalBreakfast++
				dateMap[v.Date].BreakfastTotal++
			}
			if v.Kind == enum.NursingHomeUserWithDayKindForLunch {
				totalLunch++
				dateMap[v.Date].LunchTotal++
			}
			if v.Kind == enum.NursingHomeUserWithDayKindForDinner {
				dateMap[v.Date].DinnerTotal++
				totalDinner++
			}
		}
	}

	for _, value := range dateMapKey {
		resp.List = append(resp.List, *dateMap[value])
	}

	resp.Total = total
	resp.LunchTotal = totalLunch
	resp.DinnerTotal = totalDinner
	resp.BreakfastTotal = totalBreakfast
	return
}
